### ms-efsr.idl
"""
RPC definitions for the following interfaces:
- efsrpc (v1.0): c681d488-d850-11d0-8c52-00c04fd90f7e
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRSignedIntField,
    register_dcerpc_interface,
)


class EfsRpcOpenFileRaw_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("FileName", ""),
        NDRSignedIntField("Flags", 0),
    ]


class EfsRpcOpenFileRaw_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("hContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class EfsRpcReadFileRaw_Request(NDRPacket):
    fields_desc = [NDRPacketField("hContext", NDRContextHandle(), NDRContextHandle)]


class EfsRpcReadFileRaw_Response(NDRPacket):
    fields_desc = [NDRByteField("EfsOutPipe", 0), NDRIntField("status", 0)]


class EfsRpcWriteFileRaw_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hContext", NDRContextHandle(), NDRContextHandle),
        NDRByteField("EfsInPipe", 0),
    ]


class EfsRpcWriteFileRaw_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EfsRpcCloseRaw_Request(NDRPacket):
    fields_desc = [NDRPacketField("hContext", NDRContextHandle(), NDRContextHandle)]


class EfsRpcCloseRaw_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("hContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class EfsRpcEncryptFileSrv_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("FileName", "")]


class EfsRpcEncryptFileSrv_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EfsRpcDecryptFileSrv_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("FileName", ""),
        NDRIntField("OpenFlag", 0),
    ]


class EfsRpcDecryptFileSrv_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RPC_SID_IDENTIFIER_AUTHORITY(NDRPacket):
    fields_desc = [StrFixedLenField("Value", "", length=6)]


class RPC_SID(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["SubAuthority"]
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("SubAuthorityCount", None, size_of="SubAuthority"),
        NDRPacketField(
            "IdentifierAuthority",
            RPC_SID_IDENTIFIER_AUTHORITY(),
            RPC_SID_IDENTIFIER_AUTHORITY,
        ),
        NDRConfFieldListField(
            "SubAuthority",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.SubAuthorityCount,
            conformant_in_struct=True,
        ),
    ]


class EFS_HASH_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cbData", None, size_of="bData"),
        NDRFullPointerField(
            NDRConfStrLenField("bData", "", size_is=lambda pkt: pkt.cbData),
            deferred=True,
        ),
    ]


class ENCRYPTION_CERTIFICATE_HASH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cbTotalLength", 0),
        NDRFullPointerField(
            NDRPacketField("UserSid", RPC_SID(), RPC_SID), deferred=True
        ),
        NDRFullPointerField(
            NDRPacketField("Hash", EFS_HASH_BLOB(), EFS_HASH_BLOB), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("lpDisplayInformation", ""), deferred=True
        ),
    ]


class ENCRYPTION_CERTIFICATE_HASH_LIST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("nCert_Hash", None, size_of="Users"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Users",
                [],
                ENCRYPTION_CERTIFICATE_HASH,
                size_is=lambda pkt: pkt.nCert_Hash,
                ptr_pack=True,
            ),
            deferred=True,
        ),
    ]


class EfsRpcQueryUsersOnFile_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("FileName", "")]


class EfsRpcQueryUsersOnFile_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "Users",
                ENCRYPTION_CERTIFICATE_HASH_LIST(),
                ENCRYPTION_CERTIFICATE_HASH_LIST,
            )
        ),
        NDRIntField("status", 0),
    ]


class EfsRpcQueryRecoveryAgents_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("FileName", "")]


class EfsRpcQueryRecoveryAgents_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "RecoveryAgents",
                ENCRYPTION_CERTIFICATE_HASH_LIST(),
                ENCRYPTION_CERTIFICATE_HASH_LIST,
            )
        ),
        NDRIntField("status", 0),
    ]


class EfsRpcRemoveUsersFromFile_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("FileName", ""),
        NDRPacketField(
            "Users",
            ENCRYPTION_CERTIFICATE_HASH_LIST(),
            ENCRYPTION_CERTIFICATE_HASH_LIST,
        ),
    ]


class EfsRpcRemoveUsersFromFile_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EFS_CERTIFICATE_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("dwCertEncodingType", 0),
        NDRIntField("cbData", None, size_of="bData"),
        NDRFullPointerField(
            NDRConfStrLenField("bData", "", size_is=lambda pkt: pkt.cbData),
            deferred=True,
        ),
    ]


class ENCRYPTION_CERTIFICATE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cbTotalLength", 0),
        NDRFullPointerField(
            NDRPacketField("UserSid", RPC_SID(), RPC_SID), deferred=True
        ),
        NDRFullPointerField(
            NDRPacketField("CertBlob", EFS_CERTIFICATE_BLOB(), EFS_CERTIFICATE_BLOB),
            deferred=True,
        ),
    ]


class ENCRYPTION_CERTIFICATE_LIST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("nUsers", None, size_of="Users"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Users",
                [],
                ENCRYPTION_CERTIFICATE,
                size_is=lambda pkt: pkt.nUsers,
                ptr_pack=True,
            ),
            deferred=True,
        ),
    ]


class EfsRpcAddUsersToFile_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("FileName", ""),
        NDRPacketField(
            "EncryptionCertificates",
            ENCRYPTION_CERTIFICATE_LIST(),
            ENCRYPTION_CERTIFICATE_LIST,
        ),
    ]


class EfsRpcAddUsersToFile_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EFS_RPC_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cbData", None, size_of="bData"),
        NDRFullPointerField(
            NDRConfStrLenField("bData", "", size_is=lambda pkt: pkt.cbData),
            deferred=True,
        ),
    ]


class EfsRpcNotSupported_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("Reserved1", ""),
        NDRConfVarStrNullFieldUtf16("Reserved2", ""),
        NDRIntField("dwReserved1", 0),
        NDRIntField("dwReserved2", 0),
        NDRFullPointerField(NDRPacketField("Reserved", EFS_RPC_BLOB(), EFS_RPC_BLOB)),
        NDRSignedIntField("bReserved", 0),
    ]


class EfsRpcNotSupported_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EfsRpcFileKeyInfo_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("FileName", ""),
        NDRIntField("InfoClass", 0),
    ]


class EfsRpcFileKeyInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("KeyInfo", EFS_RPC_BLOB(), EFS_RPC_BLOB)),
        NDRIntField("status", 0),
    ]


class EfsRpcDuplicateEncryptionInfoFile_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("SrcFileName", ""),
        NDRConfVarStrNullFieldUtf16("DestFileName", ""),
        NDRIntField("dwCreationDisposition", 0),
        NDRIntField("dwAttributes", 0),
        NDRFullPointerField(NDRPacketField("RelativeSD", EFS_RPC_BLOB(), EFS_RPC_BLOB)),
        NDRSignedIntField("bInheritHandle", 0),
    ]


class EfsRpcDuplicateEncryptionInfoFile_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EfsRpcAddUsersToFileEx_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwFlags", 0),
        NDRFullPointerField(NDRPacketField("Reserved", EFS_RPC_BLOB(), EFS_RPC_BLOB)),
        NDRConfVarStrNullFieldUtf16("FileName", ""),
        NDRPacketField(
            "EncryptionCertificates",
            ENCRYPTION_CERTIFICATE_LIST(),
            ENCRYPTION_CERTIFICATE_LIST,
        ),
    ]


class EfsRpcAddUsersToFileEx_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EfsRpcFileKeyInfoEx_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwFileKeyInfoFlags", 0),
        NDRFullPointerField(NDRPacketField("Reserved", EFS_RPC_BLOB(), EFS_RPC_BLOB)),
        NDRConfVarStrNullFieldUtf16("FileName", ""),
        NDRIntField("InfoClass", 0),
    ]


class EfsRpcFileKeyInfoEx_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("KeyInfo", EFS_RPC_BLOB(), EFS_RPC_BLOB)),
        NDRIntField("status", 0),
    ]


class EfsRpcGetEncryptedFileMetadata_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("FileName", "")]


class EfsRpcGetEncryptedFileMetadata_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("EfsStreamBlob", EFS_RPC_BLOB(), EFS_RPC_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class ENCRYPTED_FILE_METADATA_SIGNATURE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("dwEfsAccessType", 0),
        NDRFullPointerField(
            NDRPacketField(
                "CertificatesAdded",
                ENCRYPTION_CERTIFICATE_HASH_LIST(),
                ENCRYPTION_CERTIFICATE_HASH_LIST,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRPacketField(
                "EncryptionCertificate",
                ENCRYPTION_CERTIFICATE(),
                ENCRYPTION_CERTIFICATE,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRPacketField("EfsStreamSignature", EFS_RPC_BLOB(), EFS_RPC_BLOB),
            deferred=True,
        ),
    ]


class EfsRpcSetEncryptedFileMetadata_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("FileName", ""),
        NDRFullPointerField(
            NDRPacketField("OldEfsStreamBlob", EFS_RPC_BLOB(), EFS_RPC_BLOB)
        ),
        NDRPacketField("NewEfsStreamBlob", EFS_RPC_BLOB(), EFS_RPC_BLOB),
        NDRFullPointerField(
            NDRPacketField(
                "NewEfsSignature",
                ENCRYPTED_FILE_METADATA_SIGNATURE(),
                ENCRYPTED_FILE_METADATA_SIGNATURE,
            )
        ),
    ]


class EfsRpcSetEncryptedFileMetadata_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EfsRpcFlushEfsCache_Request(NDRPacket):
    fields_desc = []


class EfsRpcFlushEfsCache_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EfsRpcEncryptFileExSrv_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("FileName", ""),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ProtectorDescriptor", "")),
        NDRIntField("Flags", 0),
    ]


class EfsRpcEncryptFileExSrv_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class PENCRYPTION_PROTECTOR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cbTotalLength", 0),
        NDRFullPointerField(
            NDRPacketField("UserSid", RPC_SID(), RPC_SID), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("lpProtectorDescriptor", ""), deferred=True
        ),
    ]


class PENCRYPTION_PROTECTOR_LIST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("nProtectors", None, size_of="pProtectors"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pProtectors",
                [],
                PENCRYPTION_PROTECTOR,
                size_is=lambda pkt: pkt.nProtectors,
                ptr_pack=True,
            ),
            deferred=True,
        ),
    ]


class EfsRpcQueryProtectors_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("FileName", "")]


class EfsRpcQueryProtectors_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ppProtectorList",
                PENCRYPTION_PROTECTOR_LIST(),
                PENCRYPTION_PROTECTOR_LIST,
            )
        ),
        NDRIntField("status", 0),
    ]


EFSRPC_OPNUMS = {
    0: DceRpcOp(EfsRpcOpenFileRaw_Request, EfsRpcOpenFileRaw_Response),
    1: DceRpcOp(EfsRpcReadFileRaw_Request, EfsRpcReadFileRaw_Response),
    2: DceRpcOp(EfsRpcWriteFileRaw_Request, EfsRpcWriteFileRaw_Response),
    3: DceRpcOp(EfsRpcCloseRaw_Request, EfsRpcCloseRaw_Response),
    4: DceRpcOp(EfsRpcEncryptFileSrv_Request, EfsRpcEncryptFileSrv_Response),
    5: DceRpcOp(EfsRpcDecryptFileSrv_Request, EfsRpcDecryptFileSrv_Response),
    6: DceRpcOp(EfsRpcQueryUsersOnFile_Request, EfsRpcQueryUsersOnFile_Response),
    7: DceRpcOp(EfsRpcQueryRecoveryAgents_Request, EfsRpcQueryRecoveryAgents_Response),
    8: DceRpcOp(EfsRpcRemoveUsersFromFile_Request, EfsRpcRemoveUsersFromFile_Response),
    9: DceRpcOp(EfsRpcAddUsersToFile_Request, EfsRpcAddUsersToFile_Response),
    # 10: Opnum10NotUsedOnWire,
    11: DceRpcOp(EfsRpcNotSupported_Request, EfsRpcNotSupported_Response),
    12: DceRpcOp(EfsRpcFileKeyInfo_Request, EfsRpcFileKeyInfo_Response),
    13: DceRpcOp(
        EfsRpcDuplicateEncryptionInfoFile_Request,
        EfsRpcDuplicateEncryptionInfoFile_Response,
    ),
    # 14: Opnum14NotUsedOnWire,
    15: DceRpcOp(EfsRpcAddUsersToFileEx_Request, EfsRpcAddUsersToFileEx_Response),
    16: DceRpcOp(EfsRpcFileKeyInfoEx_Request, EfsRpcFileKeyInfoEx_Response),
    # 17: Opnum17NotUsedOnWire,
    18: DceRpcOp(
        EfsRpcGetEncryptedFileMetadata_Request, EfsRpcGetEncryptedFileMetadata_Response
    ),
    19: DceRpcOp(
        EfsRpcSetEncryptedFileMetadata_Request, EfsRpcSetEncryptedFileMetadata_Response
    ),
    20: DceRpcOp(EfsRpcFlushEfsCache_Request, EfsRpcFlushEfsCache_Response),
    21: DceRpcOp(EfsRpcEncryptFileExSrv_Request, EfsRpcEncryptFileExSrv_Response),
    22: DceRpcOp(EfsRpcQueryProtectors_Request, EfsRpcQueryProtectors_Response),
    # 23: Opnum23NotUsedOnWire,
    # 24: Opnum24NotUsedOnWire,
    # 25: Opnum25NotUsedOnWire,
    # 26: Opnum26NotUsedOnWire,
    # 27: Opnum27NotUsedOnWire,
    # 28: Opnum28NotUsedOnWire,
    # 29: Opnum29NotUsedOnWire,
    # 30: Opnum30NotUsedOnWire,
    # 31: Opnum31NotUsedOnWire,
    # 32: Opnum32NotUsedOnWire,
    # 33: Opnum33NotUsedOnWire,
    # 34: Opnum34NotUsedOnWire,
    # 35: Opnum35NotUsedOnWire,
    # 36: Opnum36NotUsedOnWire,
    # 37: Opnum37NotUsedOnWire,
    # 38: Opnum38NotUsedOnWire,
    # 39: Opnum39NotUsedOnWire,
    # 40: Opnum40NotUsedOnWire,
    # 41: Opnum41NotUsedOnWire,
    # 42: Opnum42NotUsedOnWire,
    # 43: Opnum43NotUsedOnWire,
    # 44: Opnum44NotUsedOnWire
}
register_dcerpc_interface(
    name="efsrpc",
    uuid=uuid.UUID("c681d488-d850-11d0-8c52-00c04fd90f7e"),
    version="1.0",
    opnums=EFSRPC_OPNUMS,
)



### ms-mqds.idl
"""
RPC definitions for the following interfaces:
- dscomm (v1.0): 77df7a80-f298-11d0-8358-00a024c480a8
- dscomm2 (v1.0): 708cca10-9569-11d1-b2a5-0060977d8118
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import PacketListField, StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarPacketListField,
    NDRConfVarStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRRecursiveField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    register_dcerpc_interface,
)


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_LPWSTR = 31
    VT_BLOB = 65
    VT_CLSID = 72
    VT_VECTOR = 4096


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class ULARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRLongField("QuadPart", 0)]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cbSize", None, size_of="pBlobData"),
        NDRFullPointerField(
            NDRConfStrLenField("pBlobData", "", size_is=lambda pkt: pkt.cbSize),
            deferred=True,
        ),
    ]


class CAUB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfStrLenField("pElems", "", size_is=lambda pkt: pkt.cElems),
            deferred=True,
        ),
    ]


class CAUI(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("pElems", "", size_is=lambda pkt: pkt.cElems),
            deferred=True,
        ),
    ]


class CAL(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "pElems",
                [],
                NDRSignedIntField("pElems", 0),
                size_is=lambda pkt: pkt.cElems,
            ),
            deferred=True,
        ),
    ]


class CAUL(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "pElems", [], NDRIntField("pElems", 0), size_is=lambda pkt: pkt.cElems
            ),
            deferred=True,
        ),
    ]


class CAUH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pElems", [], ULARGE_INTEGER, size_is=lambda pkt: pkt.cElems
            ),
            deferred=True,
        ),
    ]


class CACLSID(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfPacketListField("pElems", [], GUID, size_is=lambda pkt: pkt.cElems),
            deferred=True,
        ),
    ]


class CALPWSTR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "pElems",
                [],
                NDRFullPointerField(
                    NDRConfVarStrNullFieldUtf16("pElems", ""), deferred=True
                ),
                size_is=lambda pkt: pkt.cElems,
            ),
            deferred=True,
        ),
    ]


class tag_inner_PROPVARIANT(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRShortField("vt", 0),
        NDRByteField("wReserved1", 0),
        NDRByteField("wReserved2", 0),
        NDRIntField("wReserved3", 0),
        NDRRecursiveField("_varUnion"),
    ]


class S_DSCreateObject_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRFullPointerField(NDRShortField("pwcsPathName", 0)),
        NDRIntField("dwSDLength", None, size_of="SecurityDescriptor"),
        NDRConfStrLenField(
            "SecurityDescriptor", "", size_is=lambda pkt: pkt.dwSDLength
        ),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRFullPointerField(NDRPacketField("pObjGuid", GUID(), GUID)),
    ]


class S_DSCreateObject_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("pObjGuid", GUID(), GUID)),
        NDRIntField("status", 0),
    ]


class S_DSDeleteObject_Request(NDRPacket):
    fields_desc = [NDRIntField("dwObjectType", 0), NDRShortField("pwcsPathName", 0)]


class S_DSDeleteObject_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class S_DSGetProps_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRShortField("pwcsPathName", 0),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSGetProps_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSSetProps_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRShortField("pwcsPathName", 0),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
    ]


class S_DSSetProps_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class S_DSGetObjectSecurity_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRShortField("pwcsPathName", 0),
        NDRIntField("SecurityInformation", 0),
        NDRIntField("nLength", 0),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSGetObjectSecurity_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("pSecurityDescriptor", "", size_is=lambda pkt: pkt.nLength),
        NDRIntField("lpnLengthNeeded", 0),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSSetObjectSecurity_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRShortField("pwcsPathName", 0),
        NDRIntField("SecurityInformation", 0),
        NDRConfStrLenField("pSecurityDescriptor", "", size_is=lambda pkt: pkt.nLength),
        NDRIntField("nLength", None, size_of="pSecurityDescriptor"),
    ]


class S_DSSetObjectSecurity_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class MQPROPERTYRESTRICTION(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("rel", 0),
        NDRIntField("prop", 0),
        NDRPacketField("prval", tag_inner_PROPVARIANT(), tag_inner_PROPVARIANT),
    ]


class MQRESTRICTION(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cRes", None, size_of="paPropRes"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "paPropRes", [], MQPROPERTYRESTRICTION, size_is=lambda pkt: pkt.cRes
            ),
            deferred=True,
        ),
    ]


class MQCOLUMNSET(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cCol", None, size_of="aCol"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "aCol", [], NDRIntField("aCol", 0), size_is=lambda pkt: pkt.cCol
            ),
            deferred=True,
        ),
    ]


class MQSORTKEY(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("propColumn", 0), NDRIntField("dwOrder", 0)]


class MQSORTSET(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cCol", None, size_of="aCol"),
        NDRFullPointerField(
            NDRConfPacketListField("aCol", [], MQSORTKEY, size_is=lambda pkt: pkt.cCol),
            deferred=True,
        ),
    ]


class S_DSLookupBegin_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRShortField("pwcsContext", 0)),
        NDRFullPointerField(
            NDRPacketField("pRestriction", MQRESTRICTION(), MQRESTRICTION)
        ),
        NDRPacketField("pColumns", MQCOLUMNSET(), MQCOLUMNSET),
        NDRFullPointerField(NDRPacketField("pSort", MQSORTSET(), MQSORTSET)),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
    ]


class S_DSLookupBegin_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class S_DSLookupNext_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("Handle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwSize", 0),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSLookupNext_Response(NDRPacket):
    fields_desc = [
        NDRIntField("dwOutSize", None, size_of="pbBuffer"),
        NDRConfVarPacketListField(
            "pbBuffer",
            [],
            tag_inner_PROPVARIANT,
            size_is=lambda pkt: pkt.dwSize,
            length_is=lambda pkt: pkt.dwOutSize,
        ),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSLookupEnd_Request(NDRPacket):
    fields_desc = [NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle)]


class S_DSLookupEnd_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class S_DSDeleteObjectGuid_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRPacketField("pGuid", GUID(), GUID),
    ]


class S_DSDeleteObjectGuid_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class S_DSGetPropsGuid_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRFullPointerField(NDRPacketField("pGuid", GUID(), GUID)),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSGetPropsGuid_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSSetPropsGuid_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRPacketField("pGuid", GUID(), GUID),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
    ]


class S_DSSetPropsGuid_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class S_DSGetObjectSecurityGuid_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRPacketField("pGuid", GUID(), GUID),
        NDRIntField("SecurityInformation", 0),
        NDRIntField("nLength", 0),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSGetObjectSecurityGuid_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("pSecurityDescriptor", "", size_is=lambda pkt: pkt.nLength),
        NDRIntField("lpnLengthNeeded", 0),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSSetObjectSecurityGuid_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRPacketField("pGuid", GUID(), GUID),
        NDRIntField("SecurityInformation", 0),
        NDRConfStrLenField("pSecurityDescriptor", "", size_is=lambda pkt: pkt.nLength),
        NDRIntField("nLength", None, size_of="pSecurityDescriptor"),
    ]


class S_DSSetObjectSecurityGuid_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class S_DSQMSetMachineProperties_Request(NDRPacket):
    fields_desc = [
        NDRShortField("pwcsPathName", 0),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRIntField("dwContext", 0),
    ]


class S_DSQMSetMachineProperties_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class S_DSCreateServersCache_Request(NDRPacket):
    fields_desc = [
        NDRIntField("pdwIndex", 0),
        NDRFullPointerField(
            NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lplpSiteServers", ""))
        ),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSCreateServersCache_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwIndex", 0),
        NDRFullPointerField(
            NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lplpSiteServers", ""))
        ),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSQMSetMachinePropertiesSignProc_Request(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("abChallenge", "", size_is=lambda pkt: pkt.dwCallengeSize),
        NDRIntField("dwCallengeSize", None, size_of="abChallenge"),
        NDRIntField("dwContext", 0),
        NDRConfVarStrLenField(
            "abSignature",
            "",
            size_is=lambda pkt: pkt.dwSignatureMaxSize,
            length_is=lambda pkt: pkt.pdwSignatureSize,
        ),
        NDRIntField("pdwSignatureSize", None, size_of="abSignature"),
        NDRIntField("dwSignatureMaxSize", None, size_of="abSignature"),
    ]


class S_DSQMSetMachinePropertiesSignProc_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "abSignature",
            "",
            size_is=lambda pkt: pkt.dwSignatureMaxSize,
            length_is=lambda pkt: pkt.pdwSignatureSize,
        ),
        NDRIntField("pdwSignatureSize", None, size_of="abSignature"),
        NDRIntField("status", 0),
    ]


class S_DSQMGetObjectSecurity_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRPacketField("pGuid", GUID(), GUID),
        NDRIntField("SecurityInformation", 0),
        NDRIntField("nLength", 0),
        NDRIntField("dwContext", 0),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSQMGetObjectSecurity_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("pSecurityDescriptor", "", size_is=lambda pkt: pkt.nLength),
        NDRIntField("lpnLengthNeeded", 0),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSQMGetObjectSecurityChallengeResponceProc_Request(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("abChallenge", "", size_is=lambda pkt: pkt.dwCallengeSize),
        NDRIntField("dwCallengeSize", None, size_of="abChallenge"),
        NDRIntField("dwContext", 0),
        NDRConfVarStrLenField(
            "abCallengeResponce",
            "",
            size_is=lambda pkt: pkt.dwCallengeResponceMaxSize,
            length_is=lambda pkt: pkt.pdwCallengeResponceSize,
        ),
        NDRIntField("pdwCallengeResponceSize", None, size_of="abCallengeResponce"),
        NDRIntField("dwCallengeResponceMaxSize", None, size_of="abCallengeResponce"),
    ]


class S_DSQMGetObjectSecurityChallengeResponceProc_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "abCallengeResponce",
            "",
            size_is=lambda pkt: pkt.dwCallengeResponceMaxSize,
            length_is=lambda pkt: pkt.pdwCallengeResponceSize,
        ),
        NDRIntField("pdwCallengeResponceSize", None, size_of="abCallengeResponce"),
        NDRIntField("status", 0),
    ]


class S_InitSecCtx_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwContext", 0),
        NDRConfStrLenField("pServerbuff", "", size_is=lambda pkt: pkt.dwServerBuffSize),
        NDRIntField("dwServerBuffSize", None, size_of="pServerbuff"),
        NDRIntField("dwClientBuffMaxSize", 0),
    ]


class S_InitSecCtx_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "pClientBuff",
            "",
            size_is=lambda pkt: pkt.dwClientBuffMaxSize,
            length_is=lambda pkt: pkt.pdwClientBuffSize,
        ),
        NDRIntField("pdwClientBuffSize", None, size_of="pClientBuff"),
        NDRIntField("status", 0),
    ]


class S_DSValidateServer_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pguidEnterpriseId", GUID(), GUID),
        NDRSignedIntField("fSetupMode", 0),
        NDRIntField("dwContext", 0),
        NDRIntField("dwClientBuffMaxSize", None, size_of="pClientBuff"),
        NDRConfVarStrLenField(
            "pClientBuff",
            "",
            size_is=lambda pkt: pkt.dwClientBuffMaxSize,
            length_is=lambda pkt: pkt.dwClientBuffSize,
        ),
        NDRIntField("dwClientBuffSize", None, size_of="pClientBuff"),
    ]


class S_DSValidateServer_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class S_DSCloseServerHandle_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pphServerAuth", NDRContextHandle(), NDRContextHandle)
    ]


class S_DSCloseServerHandle_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class S_DSGetServerPort_Request(NDRPacket):
    fields_desc = [NDRIntField("fIP", 0)]


class S_DSGetServerPort_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


DSCOMM_OPNUMS = {
    0: DceRpcOp(S_DSCreateObject_Request, S_DSCreateObject_Response),
    1: DceRpcOp(S_DSDeleteObject_Request, S_DSDeleteObject_Response),
    2: DceRpcOp(S_DSGetProps_Request, S_DSGetProps_Response),
    3: DceRpcOp(S_DSSetProps_Request, S_DSSetProps_Response),
    4: DceRpcOp(S_DSGetObjectSecurity_Request, S_DSGetObjectSecurity_Response),
    5: DceRpcOp(S_DSSetObjectSecurity_Request, S_DSSetObjectSecurity_Response),
    6: DceRpcOp(S_DSLookupBegin_Request, S_DSLookupBegin_Response),
    7: DceRpcOp(S_DSLookupNext_Request, S_DSLookupNext_Response),
    8: DceRpcOp(S_DSLookupEnd_Request, S_DSLookupEnd_Response),
    # 9: Opnum9NotUsedOnWire,
    10: DceRpcOp(S_DSDeleteObjectGuid_Request, S_DSDeleteObjectGuid_Response),
    11: DceRpcOp(S_DSGetPropsGuid_Request, S_DSGetPropsGuid_Response),
    12: DceRpcOp(S_DSSetPropsGuid_Request, S_DSSetPropsGuid_Response),
    13: DceRpcOp(S_DSGetObjectSecurityGuid_Request, S_DSGetObjectSecurityGuid_Response),
    14: DceRpcOp(S_DSSetObjectSecurityGuid_Request, S_DSSetObjectSecurityGuid_Response),
    # 15: Opnum15NotUsedOnWire,
    # 16: Opnum16NotUsedOnWire,
    # 17: Opnum17NotUsedOnWire,
    # 18: Opnum18NotUsedOnWire,
    19: DceRpcOp(
        S_DSQMSetMachineProperties_Request, S_DSQMSetMachineProperties_Response
    ),
    20: DceRpcOp(S_DSCreateServersCache_Request, S_DSCreateServersCache_Response),
    21: DceRpcOp(
        S_DSQMSetMachinePropertiesSignProc_Request,
        S_DSQMSetMachinePropertiesSignProc_Response,
    ),
    22: DceRpcOp(S_DSQMGetObjectSecurity_Request, S_DSQMGetObjectSecurity_Response),
    23: DceRpcOp(
        S_DSQMGetObjectSecurityChallengeResponceProc_Request,
        S_DSQMGetObjectSecurityChallengeResponceProc_Response,
    ),
    24: DceRpcOp(S_InitSecCtx_Request, S_InitSecCtx_Response),
    25: DceRpcOp(S_DSValidateServer_Request, S_DSValidateServer_Response),
    26: DceRpcOp(S_DSCloseServerHandle_Request, S_DSCloseServerHandle_Response),
    # 27: Opnum24NotUsedOnWire,
    # 28: Opnum25NotUsedOnWire,
    # 29: Opnum26NotUsedOnWire,
    30: DceRpcOp(S_DSGetServerPort_Request, S_DSGetServerPort_Response),
}
register_dcerpc_interface(
    name="dscomm",
    uuid=uuid.UUID("77df7a80-f298-11d0-8358-00a024c480a8"),
    version="1.0",
    opnums=DSCOMM_OPNUMS,
)


class S_DSGetComputerSites_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRShortField("pwcsPathName", 0)),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSGetComputerSites_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwNumberOfSites", None, size_of="ppguidSites"),
        NDRConfVarPacketListField(
            "ppguidSites",
            [],
            GUID,
            size_is=lambda pkt: pkt.pdwNumberOfSites,
            length_is=lambda pkt: pkt.pdwNumberOfSites,
            ptr_pack=True,
        ),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSGetPropsEx_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRShortField("pwcsPathName", 0),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSGetPropsEx_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSGetPropsGuidEx_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRFullPointerField(NDRPacketField("pGuid", GUID(), GUID)),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSGetPropsGuidEx_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


class S_DSBeginDeleteNotification_Request(NDRPacket):
    fields_desc = [
        NDRShortField("pwcsPathName", 0),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
    ]


class S_DSBeginDeleteNotification_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class S_DSNotifyDelete_Request(NDRPacket):
    fields_desc = [NDRPacketField("Handle", NDRContextHandle(), NDRContextHandle)]


class S_DSNotifyDelete_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class S_DSEndDeleteNotification_Request(NDRPacket):
    fields_desc = [NDRPacketField("pHandle", NDRContextHandle(), NDRContextHandle)]


class S_DSEndDeleteNotification_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class S_DSIsServerGC_Request(NDRPacket):
    fields_desc = []


class S_DSIsServerGC_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class S_DSGetGCListInDomain_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRShortField("lpwszComputerName", 0)),
        NDRFullPointerField(NDRShortField("lpwszDomainName", 0)),
        NDRPacketField("phServerAuth", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwServerSignatureSize", 0),
    ]


class S_DSGetGCListInDomain_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lplpwszGCList", "")),
        NDRConfStrLenField(
            "pbServerSignature", "", size_is=lambda pkt: pkt.pdwServerSignatureSize
        ),
        NDRIntField("pdwServerSignatureSize", None, size_of="pbServerSignature"),
        NDRIntField("status", 0),
    ]


DSCOMM2_OPNUMS = {
    0: DceRpcOp(S_DSGetComputerSites_Request, S_DSGetComputerSites_Response),
    1: DceRpcOp(S_DSGetPropsEx_Request, S_DSGetPropsEx_Response),
    2: DceRpcOp(S_DSGetPropsGuidEx_Request, S_DSGetPropsGuidEx_Response),
    3: DceRpcOp(
        S_DSBeginDeleteNotification_Request, S_DSBeginDeleteNotification_Response
    ),
    4: DceRpcOp(S_DSNotifyDelete_Request, S_DSNotifyDelete_Response),
    5: DceRpcOp(S_DSEndDeleteNotification_Request, S_DSEndDeleteNotification_Response),
    6: DceRpcOp(S_DSIsServerGC_Request, S_DSIsServerGC_Response),
    # 7: Opnum7NotUsedOnWire,
    8: DceRpcOp(S_DSGetGCListInDomain_Request, S_DSGetGCListInDomain_Response),
}
register_dcerpc_interface(
    name="dscomm2",
    uuid=uuid.UUID("708cca10-9569-11d1-b2a5-0060977d8118"),
    version="1.0",
    opnums=DSCOMM2_OPNUMS,
)



### ms-cmrp_clusapi2.idl
"""
RPC definitions for the following interfaces:
- clusapi (v2.0): b97db8b2-4c63-11cf-bff6-08002be23f2f
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import PacketListField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarPacketListField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    register_dcerpc_interface,
)


class ApiOpenCluster_Request(NDRPacket):
    fields_desc = []


class ApiOpenCluster_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class ApiCloseCluster_Request(NDRPacket):
    fields_desc = [NDRPacketField("Cluster", NDRContextHandle(), NDRContextHandle)]


class ApiCloseCluster_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("Cluster", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ApiSetClusterName_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("NewClusterName", "")]


class ApiSetClusterName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiGetClusterName_Request(NDRPacket):
    fields_desc = []


class ApiGetClusterName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ClusterName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("NodeName", "")),
        NDRIntField("status", 0),
    ]


class ApiGetClusterVersion_Request(NDRPacket):
    fields_desc = []


class ApiGetClusterVersion_Response(NDRPacket):
    fields_desc = [
        NDRShortField("lpwMajorVersion", 0),
        NDRShortField("lpwMinorVersion", 0),
        NDRShortField("lpwBuildNumber", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lpszVendorId", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lpszCSDVersion", "")),
        NDRIntField("status", 0),
    ]


class ApiGetQuorumResource_Request(NDRPacket):
    fields_desc = []


class ApiGetQuorumResource_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lpszResourceName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lpszDeviceName", "")),
        NDRIntField("pdwMaxQuorumLogSize", 0),
        NDRIntField("status", 0),
    ]


class ApiSetQuorumResource_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpszDeviceName", ""),
        NDRIntField("dwMaxQuorumLogSize", 0),
    ]


class ApiSetQuorumResource_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ENUM_ENTRY(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Type", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("Name", ""), deferred=True),
    ]


class PENUM_LIST(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["Entry"]
    fields_desc = [
        NDRIntField("EntryCount", None, size_of="Entry"),
        NDRConfPacketListField(
            "Entry",
            [],
            ENUM_ENTRY,
            size_is=lambda pkt: pkt.EntryCount,
            conformant_in_struct=True,
        ),
    ]


class ApiCreateEnum_Request(NDRPacket):
    fields_desc = [NDRIntField("dwType", 0)]


class ApiCreateEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ReturnEnum", PENUM_LIST(), PENUM_LIST)),
        NDRIntField("status", 0),
    ]


class ApiOpenResource_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("lpszResourceName", "")]


class ApiOpenResource_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class ApiCreateResource_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpszResourceName", ""),
        NDRConfVarStrNullFieldUtf16("lpszResourceType", ""),
        NDRIntField("dwFlags", 0),
    ]


class ApiCreateResource_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class ApiDeleteResource_Request(NDRPacket):
    fields_desc = [NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle)]


class ApiDeleteResource_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiCloseResource_Request(NDRPacket):
    fields_desc = [NDRPacketField("Resource", NDRContextHandle(), NDRContextHandle)]


class ApiCloseResource_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("Resource", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ApiGetResourceState_Request(NDRPacket):
    fields_desc = [NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle)]


class ApiGetResourceState_Response(NDRPacket):
    fields_desc = [
        NDRIntField("State", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("NodeName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("GroupName", "")),
        NDRIntField("status", 0),
    ]


class ApiSetResourceName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpszResourceName", ""),
    ]


class ApiSetResourceName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiGetResourceId_Request(NDRPacket):
    fields_desc = [NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle)]


class ApiGetResourceId_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pGuid", "")),
        NDRIntField("status", 0),
    ]


class ApiGetResourceType_Request(NDRPacket):
    fields_desc = [NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle)]


class ApiGetResourceType_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lpszResourceType", "")),
        NDRIntField("status", 0),
    ]


class ApiFailResource_Request(NDRPacket):
    fields_desc = [NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle)]


class ApiFailResource_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiOnlineResource_Request(NDRPacket):
    fields_desc = [NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle)]


class ApiOnlineResource_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiOfflineResource_Request(NDRPacket):
    fields_desc = [NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle)]


class ApiOfflineResource_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiAddResourceDependency_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hDependsOn", NDRContextHandle(), NDRContextHandle),
    ]


class ApiAddResourceDependency_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiRemoveResourceDependency_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hDependsOn", NDRContextHandle(), NDRContextHandle),
    ]


class ApiRemoveResourceDependency_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiCanResourceBeDependent_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hResourceDependent", NDRContextHandle(), NDRContextHandle),
    ]


class ApiCanResourceBeDependent_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiCreateResEnum_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwType", 0),
    ]


class ApiCreateResEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ReturnEnum", PENUM_LIST(), PENUM_LIST)),
        NDRIntField("status", 0),
    ]


class ApiAddResourceNode_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
    ]


class ApiAddResourceNode_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiRemoveResourceNode_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
    ]


class ApiRemoveResourceNode_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiChangeResourceGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
    ]


class ApiChangeResourceGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiCreateResourceType_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("lpszTypeName", ""),
        NDRConfVarStrNullFieldUtf16("lpszDisplayName", ""),
        NDRConfVarStrNullFieldUtf16("lpszDllName", ""),
        NDRIntField("dwLooksAlive", 0),
        NDRIntField("dwIsAlive", 0),
    ]


class ApiCreateResourceType_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiDeleteResourceType_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("lpszTypeName", "")]


class ApiDeleteResourceType_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiGetRootKey_Request(NDRPacket):
    fields_desc = [NDRIntField("samDesired", 0)]


class ApiGetRootKey_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class RPC_SECURITY_DESCRIPTOR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrLenField(
                "lpSecurityDescriptor",
                "",
                size_is=lambda pkt: pkt.cbInSecurityDescriptor,
                length_is=lambda pkt: pkt.cbOutSecurityDescriptor,
            ),
            deferred=True,
        ),
        NDRIntField("cbInSecurityDescriptor", None, size_of="lpSecurityDescriptor"),
        NDRIntField("cbOutSecurityDescriptor", None, size_of="lpSecurityDescriptor"),
    ]


class PRPC_SECURITY_ATTRIBUTES(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("nLength", 0),
        NDRPacketField(
            "RpcSecurityDescriptor", RPC_SECURITY_DESCRIPTOR(), RPC_SECURITY_DESCRIPTOR
        ),
        NDRSignedIntField("bInheritHandle", 0),
    ]


class ApiCreateKey_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpSubKey", ""),
        NDRIntField("dwOptions", 0),
        NDRIntField("samDesired", 0),
        NDRFullPointerField(
            NDRPacketField(
                "lpSecurityAttributes",
                PRPC_SECURITY_ATTRIBUTES(),
                PRPC_SECURITY_ATTRIBUTES,
            )
        ),
    ]


class ApiCreateKey_Response(NDRPacket):
    fields_desc = [NDRIntField("lpdwDisposition", 0), NDRIntField("Status", 0)]


class ApiOpenKey_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpSubKey", ""),
        NDRIntField("samDesired", 0),
    ]


class ApiOpenKey_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class FILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class ApiEnumKey_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwIndex", 0),
    ]


class ApiEnumKey_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("KeyName", "")),
        NDRPacketField("lpftLastWriteTime", FILETIME(), FILETIME),
        NDRIntField("status", 0),
    ]


class ApiSetValue_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpValueName", ""),
        NDRIntField("dwType", 0),
        NDRConfStrLenField("lpData", "", size_is=lambda pkt: pkt.cbData),
        NDRIntField("cbData", None, size_of="lpData"),
    ]


class ApiSetValue_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiDeleteValue_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpValueName", ""),
    ]


class ApiDeleteValue_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiQueryValue_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpValueName", ""),
        NDRIntField("cbData", 0),
    ]


class ApiQueryValue_Response(NDRPacket):
    fields_desc = [
        NDRIntField("lpValueType", 0),
        NDRConfStrLenField("lpData", "", size_is=lambda pkt: pkt.cbData),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiDeleteKey_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpSubKey", ""),
    ]


class ApiDeleteKey_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiEnumValue_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwIndex", 0),
        NDRIntField("lpcbData", 0),
    ]


class ApiEnumValue_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lpValueName", "")),
        NDRIntField("lpType", 0),
        NDRConfStrLenField("lpData", "", size_is=lambda pkt: pkt.lpcbData),
        NDRIntField("lpcbData", None, size_of="lpData"),
        NDRIntField("TotalSize", 0),
        NDRIntField("status", 0),
    ]


class ApiCloseKey_Request(NDRPacket):
    fields_desc = [NDRPacketField("pKey", NDRContextHandle(), NDRContextHandle)]


class ApiCloseKey_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pKey", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class PFILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class ApiQueryInfoKey_Request(NDRPacket):
    fields_desc = [NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle)]


class ApiQueryInfoKey_Response(NDRPacket):
    fields_desc = [
        NDRIntField("lpcSubKeys", 0),
        NDRIntField("lpcbMaxSubKeyLen", 0),
        NDRIntField("lpcValues", 0),
        NDRIntField("lpcbMaxValueNameLen", 0),
        NDRIntField("lpcbMaxValueLen", 0),
        NDRIntField("lpcbSecurityDescriptor", 0),
        NDRPacketField("lpftLastWriteTime", PFILETIME(), PFILETIME),
        NDRIntField("status", 0),
    ]


class PRPC_SECURITY_DESCRIPTOR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrLenField(
                "lpSecurityDescriptor",
                "",
                size_is=lambda pkt: pkt.cbInSecurityDescriptor,
                length_is=lambda pkt: pkt.cbOutSecurityDescriptor,
            ),
            deferred=True,
        ),
        NDRIntField("cbInSecurityDescriptor", None, size_of="lpSecurityDescriptor"),
        NDRIntField("cbOutSecurityDescriptor", None, size_of="lpSecurityDescriptor"),
    ]


class ApiSetKeySecurity_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRIntField("SecurityInformation", 0),
        NDRPacketField(
            "pRpcSecurityDescriptor",
            PRPC_SECURITY_DESCRIPTOR(),
            PRPC_SECURITY_DESCRIPTOR,
        ),
    ]


class ApiSetKeySecurity_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiGetKeySecurity_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRIntField("SecurityInformation", 0),
        NDRPacketField(
            "pRpcSecurityDescriptor",
            PRPC_SECURITY_DESCRIPTOR(),
            PRPC_SECURITY_DESCRIPTOR,
        ),
    ]


class ApiGetKeySecurity_Response(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "pRpcSecurityDescriptor",
            PRPC_SECURITY_DESCRIPTOR(),
            PRPC_SECURITY_DESCRIPTOR,
        ),
        NDRIntField("status", 0),
    ]


class ApiOpenGroup_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("lpszGroupName", "")]


class ApiOpenGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class ApiCreateGroup_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("lpszGroupName", "")]


class ApiCreateGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class ApiDeleteGroup_Request(NDRPacket):
    fields_desc = [NDRPacketField("Group", NDRContextHandle(), NDRContextHandle)]


class ApiDeleteGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiCloseGroup_Request(NDRPacket):
    fields_desc = [NDRPacketField("Group", NDRContextHandle(), NDRContextHandle)]


class ApiCloseGroup_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("Group", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ApiGetGroupState_Request(NDRPacket):
    fields_desc = [NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle)]


class ApiGetGroupState_Response(NDRPacket):
    fields_desc = [
        NDRIntField("State", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("NodeName", "")),
        NDRIntField("status", 0),
    ]


class ApiSetGroupName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpszGroupName", ""),
    ]


class ApiSetGroupName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiGetGroupId_Request(NDRPacket):
    fields_desc = [NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle)]


class ApiGetGroupId_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pGuid", "")),
        NDRIntField("status", 0),
    ]


class ApiGetNodeId_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle)]


class ApiGetNodeId_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pGuid", "")),
        NDRIntField("status", 0),
    ]


class ApiOnlineGroup_Request(NDRPacket):
    fields_desc = [NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle)]


class ApiOnlineGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiOfflineGroup_Request(NDRPacket):
    fields_desc = [NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle)]


class ApiOfflineGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiMoveGroup_Request(NDRPacket):
    fields_desc = [NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle)]


class ApiMoveGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiMoveGroupToNode_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
    ]


class ApiMoveGroupToNode_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiCreateGroupResourceEnum_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwType", 0),
    ]


class ApiCreateGroupResourceEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ReturnEnum", PENUM_LIST(), PENUM_LIST)),
        NDRIntField("status", 0),
    ]


class ApiSetGroupNodeList_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
        NDRConfStrLenField("lpNodeList", "", size_is=lambda pkt: pkt.cbListSize),
        NDRIntField("cbListSize", None, size_of="lpNodeList"),
    ]


class ApiSetGroupNodeList_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiCreateNotify_Request(NDRPacket):
    fields_desc = []


class ApiCreateNotify_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class ApiCloseNotify_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle)]


class ApiCloseNotify_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ApiAddNotifyCluster_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hCluster", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
    ]


class ApiAddNotifyCluster_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiAddNotifyNode_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
    ]


class ApiAddNotifyNode_Response(NDRPacket):
    fields_desc = [NDRIntField("dwStateSequence", 0), NDRIntField("status", 0)]


class ApiAddNotifyGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
    ]


class ApiAddNotifyGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("dwStateSequence", 0), NDRIntField("status", 0)]


class ApiAddNotifyResource_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
    ]


class ApiAddNotifyResource_Response(NDRPacket):
    fields_desc = [NDRIntField("dwStateSequence", 0), NDRIntField("status", 0)]


class ApiAddNotifyKey_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hKey", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwNotifyKey", 0),
        NDRIntField("dwFilter", 0),
        NDRSignedIntField("WatchSubTree", 0),
    ]


class ApiAddNotifyKey_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiReAddNotifyNode_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
        NDRIntField("StateSequence", 0),
    ]


class ApiReAddNotifyNode_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiReAddNotifyGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
        NDRIntField("StateSequence", 0),
    ]


class ApiReAddNotifyGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiReAddNotifyResource_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
        NDRIntField("StateSequence", 0),
    ]


class ApiReAddNotifyResource_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiGetNotify_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Timeout", 0),
    ]


class ApiGetNotify_Response(NDRPacket):
    fields_desc = [
        NDRIntField("dwNotifyKey", 0),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwStateSequence", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("Name", "")),
        NDRIntField("status", 0),
    ]


class ApiOpenNode_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("lpszNodeName", "")]


class ApiOpenNode_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class ApiCloseNode_Request(NDRPacket):
    fields_desc = [NDRPacketField("Node", NDRContextHandle(), NDRContextHandle)]


class ApiCloseNode_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("Node", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ApiGetNodeState_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle)]


class ApiGetNodeState_Response(NDRPacket):
    fields_desc = [NDRIntField("State", 0), NDRIntField("status", 0)]


class ApiPauseNode_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle)]


class ApiPauseNode_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiResumeNode_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle)]


class ApiResumeNode_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiEvictNode_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle)]


class ApiEvictNode_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiNodeResourceControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNodeResourceControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiResourceControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hResource", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiResourceControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiNodeResourceTypeControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hCluster", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpszResourceTypeName", ""),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNodeResourceTypeControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiResourceTypeControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hCluster", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpszResourceTypeName", ""),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiResourceTypeControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiNodeGroupControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNodeGroupControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiGroupControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hGroup", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiGroupControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiNodeNodeControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hHostNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNodeNodeControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiNodeControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNodeControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiOpenNetwork_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("lpszNetworkName", "")]


class ApiOpenNetwork_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class ApiCloseNetwork_Request(NDRPacket):
    fields_desc = [NDRPacketField("Network", NDRContextHandle(), NDRContextHandle)]


class ApiCloseNetwork_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("Network", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ApiGetNetworkState_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNetwork", NDRContextHandle(), NDRContextHandle)]


class ApiGetNetworkState_Response(NDRPacket):
    fields_desc = [NDRIntField("State", 0), NDRIntField("status", 0)]


class ApiSetNetworkName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNetwork", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("lpszNetworkName", ""),
    ]


class ApiSetNetworkName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiCreateNetworkEnum_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNetwork", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwType", 0),
    ]


class ApiCreateNetworkEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ReturnEnum", PENUM_LIST(), PENUM_LIST)),
        NDRIntField("status", 0),
    ]


class ApiGetNetworkId_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNetwork", NDRContextHandle(), NDRContextHandle)]


class ApiGetNetworkId_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pGuid", "")),
        NDRIntField("status", 0),
    ]


class ApiSetNetworkPriorityOrder_Request(NDRPacket):
    fields_desc = [
        NDRIntField("NetworkCount", None, size_of="NetworkIdList"),
        NDRConfVarStrLenFieldUtf16(
            "NetworkIdList", "", size_is=lambda pkt: pkt.NetworkCount
        ),
    ]


class ApiSetNetworkPriorityOrder_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiNodeNetworkControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNetwork", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNodeNetworkControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiNetworkControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNetwork", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNetworkControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiAddNotifyNetwork_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNetwork", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
    ]


class ApiAddNotifyNetwork_Response(NDRPacket):
    fields_desc = [NDRIntField("dwStateSequence", 0), NDRIntField("status", 0)]


class ApiReAddNotifyNetwork_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNetwork", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
        NDRIntField("StateSequence", 0),
    ]


class ApiReAddNotifyNetwork_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiOpenNetInterface_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("lpszNetInterfaceName", "")]


class ApiOpenNetInterface_Response(NDRPacket):
    fields_desc = [NDRIntField("Status", 0)]


class ApiCloseNetInterface_Request(NDRPacket):
    fields_desc = [NDRPacketField("NetInterface", NDRContextHandle(), NDRContextHandle)]


class ApiCloseNetInterface_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("NetInterface", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ApiGetNetInterfaceState_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNetInterface", NDRContextHandle(), NDRContextHandle)
    ]


class ApiGetNetInterfaceState_Response(NDRPacket):
    fields_desc = [NDRIntField("State", 0), NDRIntField("status", 0)]


class ApiGetNetInterface_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("lpszNodeName", ""),
        NDRConfVarStrNullFieldUtf16("lpszNetworkName", ""),
    ]


class ApiGetNetInterface_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lppszInterfaceName", "")),
        NDRIntField("status", 0),
    ]


class ApiGetNetInterfaceId_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNetInterface", NDRContextHandle(), NDRContextHandle)
    ]


class ApiGetNetInterfaceId_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pGuid", "")),
        NDRIntField("status", 0),
    ]


class ApiNodeNetInterfaceControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNetInterface", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNodeNetInterfaceControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiNetInterfaceControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNetInterface", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNetInterfaceControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiAddNotifyNetInterface_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNetInterface", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
    ]


class ApiAddNotifyNetInterface_Response(NDRPacket):
    fields_desc = [NDRIntField("dwStateSequence", 0), NDRIntField("status", 0)]


class ApiReAddNotifyNetInterface_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hNetInterface", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFilter", 0),
        NDRIntField("dwNotifyKey", 0),
        NDRIntField("StateSequence", 0),
    ]


class ApiReAddNotifyNetInterface_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiCreateNodeEnum_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwType", 0),
    ]


class ApiCreateNodeEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ReturnEnum", PENUM_LIST(), PENUM_LIST)),
        NDRIntField("status", 0),
    ]


class PCLUSTER_OPERATIONAL_VERSION_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("dwSize", 0),
        NDRIntField("dwClusterHighestVersion", 0),
        NDRIntField("dwClusterLowestVersion", 0),
        NDRIntField("dwFlags", 0),
        NDRIntField("dwReserved", 0),
    ]


class ApiGetClusterVersion2_Request(NDRPacket):
    fields_desc = []


class ApiGetClusterVersion2_Response(NDRPacket):
    fields_desc = [
        NDRShortField("lpwMajorVersion", 0),
        NDRShortField("lpwMinorVersion", 0),
        NDRShortField("lpwBuildNumber", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lpszVendorId", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lpszCSDVersion", "")),
        NDRFullPointerField(
            NDRPacketField(
                "ppClusterOpVerInfo",
                PCLUSTER_OPERATIONAL_VERSION_INFO(),
                PCLUSTER_OPERATIONAL_VERSION_INFO,
            )
        ),
        NDRIntField("status", 0),
    ]


class ApiCreateResTypeEnum_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("lpszTypeName", ""),
        NDRIntField("dwType", 0),
    ]


class ApiCreateResTypeEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ReturnEnum", PENUM_LIST(), PENUM_LIST)),
        NDRIntField("status", 0),
    ]


class ApiBackupClusterDatabase_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("lpszPathName", "")]


class ApiBackupClusterDatabase_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ApiNodeClusterControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hCluster", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hHostNode", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiNodeClusterControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiClusterControl_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hCluster", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwControlCode", 0),
        NDRConfStrLenField("lpInBuffer", "", size_is=lambda pkt: pkt.nInBufferSize),
        NDRIntField("nInBufferSize", None, size_of="lpInBuffer"),
        NDRIntField("nOutBufferSize", 0),
    ]


class ApiClusterControl_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "lpOutBuffer",
            "",
            size_is=lambda pkt: pkt.nOutBufferSize,
            length_is=lambda pkt: pkt.lpBytesReturned,
        ),
        NDRIntField("lpBytesReturned", None, size_of="lpOutBuffer"),
        NDRIntField("lpcbRequired", 0),
        NDRIntField("status", 0),
    ]


class ApiUnblockGetNotifyCall_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle)]


class ApiUnblockGetNotifyCall_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class IDL_CLUSTER_SET_PASSWORD_FLAGS(IntEnum):
    IDL_CLUSTER_SET_PASSWORD_IGNORE_DOWN_NODES = 1


class PIDL_CLUSTER_SET_PASSWORD_STATUS(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("NodeId", 0),
        NDRByteField("SetAttempted", 0),
        NDRIntField("ReturnStatus", 0),
    ]


class ApiSetServiceAccountPassword_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("lpszNewPassword", ""),
        NDRInt3264EnumField("dwFlags", 0, IDL_CLUSTER_SET_PASSWORD_FLAGS),
        NDRIntField("ReturnStatusBufferSize", 0),
    ]


class ApiSetServiceAccountPassword_Response(NDRPacket):
    fields_desc = [
        NDRConfVarPacketListField(
            "ReturnStatusBufferPtr",
            [],
            PIDL_CLUSTER_SET_PASSWORD_STATUS,
            size_is=lambda pkt: pkt.ReturnStatusBufferSize,
            length_is=lambda pkt: pkt.SizeReturned,
            ptr_pack=True,
        ),
        NDRIntField("SizeReturned", None, size_of="ReturnStatusBufferPtr"),
        NDRIntField("ExpectedBufferSize", 0),
        NDRIntField("status", 0),
    ]


CLUSAPI_OPNUMS = {
    0: DceRpcOp(ApiOpenCluster_Request, ApiOpenCluster_Response),
    1: DceRpcOp(ApiCloseCluster_Request, ApiCloseCluster_Response),
    2: DceRpcOp(ApiSetClusterName_Request, ApiSetClusterName_Response),
    3: DceRpcOp(ApiGetClusterName_Request, ApiGetClusterName_Response),
    4: DceRpcOp(ApiGetClusterVersion_Request, ApiGetClusterVersion_Response),
    5: DceRpcOp(ApiGetQuorumResource_Request, ApiGetQuorumResource_Response),
    6: DceRpcOp(ApiSetQuorumResource_Request, ApiSetQuorumResource_Response),
    7: DceRpcOp(ApiCreateEnum_Request, ApiCreateEnum_Response),
    8: DceRpcOp(ApiOpenResource_Request, ApiOpenResource_Response),
    9: DceRpcOp(ApiCreateResource_Request, ApiCreateResource_Response),
    10: DceRpcOp(ApiDeleteResource_Request, ApiDeleteResource_Response),
    11: DceRpcOp(ApiCloseResource_Request, ApiCloseResource_Response),
    12: DceRpcOp(ApiGetResourceState_Request, ApiGetResourceState_Response),
    13: DceRpcOp(ApiSetResourceName_Request, ApiSetResourceName_Response),
    14: DceRpcOp(ApiGetResourceId_Request, ApiGetResourceId_Response),
    15: DceRpcOp(ApiGetResourceType_Request, ApiGetResourceType_Response),
    16: DceRpcOp(ApiFailResource_Request, ApiFailResource_Response),
    17: DceRpcOp(ApiOnlineResource_Request, ApiOnlineResource_Response),
    18: DceRpcOp(ApiOfflineResource_Request, ApiOfflineResource_Response),
    19: DceRpcOp(ApiAddResourceDependency_Request, ApiAddResourceDependency_Response),
    20: DceRpcOp(
        ApiRemoveResourceDependency_Request, ApiRemoveResourceDependency_Response
    ),
    21: DceRpcOp(ApiCanResourceBeDependent_Request, ApiCanResourceBeDependent_Response),
    22: DceRpcOp(ApiCreateResEnum_Request, ApiCreateResEnum_Response),
    23: DceRpcOp(ApiAddResourceNode_Request, ApiAddResourceNode_Response),
    24: DceRpcOp(ApiRemoveResourceNode_Request, ApiRemoveResourceNode_Response),
    25: DceRpcOp(ApiChangeResourceGroup_Request, ApiChangeResourceGroup_Response),
    26: DceRpcOp(ApiCreateResourceType_Request, ApiCreateResourceType_Response),
    27: DceRpcOp(ApiDeleteResourceType_Request, ApiDeleteResourceType_Response),
    28: DceRpcOp(ApiGetRootKey_Request, ApiGetRootKey_Response),
    29: DceRpcOp(ApiCreateKey_Request, ApiCreateKey_Response),
    30: DceRpcOp(ApiOpenKey_Request, ApiOpenKey_Response),
    31: DceRpcOp(ApiEnumKey_Request, ApiEnumKey_Response),
    32: DceRpcOp(ApiSetValue_Request, ApiSetValue_Response),
    33: DceRpcOp(ApiDeleteValue_Request, ApiDeleteValue_Response),
    34: DceRpcOp(ApiQueryValue_Request, ApiQueryValue_Response),
    35: DceRpcOp(ApiDeleteKey_Request, ApiDeleteKey_Response),
    36: DceRpcOp(ApiEnumValue_Request, ApiEnumValue_Response),
    37: DceRpcOp(ApiCloseKey_Request, ApiCloseKey_Response),
    38: DceRpcOp(ApiQueryInfoKey_Request, ApiQueryInfoKey_Response),
    39: DceRpcOp(ApiSetKeySecurity_Request, ApiSetKeySecurity_Response),
    40: DceRpcOp(ApiGetKeySecurity_Request, ApiGetKeySecurity_Response),
    41: DceRpcOp(ApiOpenGroup_Request, ApiOpenGroup_Response),
    42: DceRpcOp(ApiCreateGroup_Request, ApiCreateGroup_Response),
    43: DceRpcOp(ApiDeleteGroup_Request, ApiDeleteGroup_Response),
    44: DceRpcOp(ApiCloseGroup_Request, ApiCloseGroup_Response),
    45: DceRpcOp(ApiGetGroupState_Request, ApiGetGroupState_Response),
    46: DceRpcOp(ApiSetGroupName_Request, ApiSetGroupName_Response),
    47: DceRpcOp(ApiGetGroupId_Request, ApiGetGroupId_Response),
    48: DceRpcOp(ApiGetNodeId_Request, ApiGetNodeId_Response),
    49: DceRpcOp(ApiOnlineGroup_Request, ApiOnlineGroup_Response),
    50: DceRpcOp(ApiOfflineGroup_Request, ApiOfflineGroup_Response),
    51: DceRpcOp(ApiMoveGroup_Request, ApiMoveGroup_Response),
    52: DceRpcOp(ApiMoveGroupToNode_Request, ApiMoveGroupToNode_Response),
    53: DceRpcOp(
        ApiCreateGroupResourceEnum_Request, ApiCreateGroupResourceEnum_Response
    ),
    54: DceRpcOp(ApiSetGroupNodeList_Request, ApiSetGroupNodeList_Response),
    55: DceRpcOp(ApiCreateNotify_Request, ApiCreateNotify_Response),
    56: DceRpcOp(ApiCloseNotify_Request, ApiCloseNotify_Response),
    57: DceRpcOp(ApiAddNotifyCluster_Request, ApiAddNotifyCluster_Response),
    58: DceRpcOp(ApiAddNotifyNode_Request, ApiAddNotifyNode_Response),
    59: DceRpcOp(ApiAddNotifyGroup_Request, ApiAddNotifyGroup_Response),
    60: DceRpcOp(ApiAddNotifyResource_Request, ApiAddNotifyResource_Response),
    61: DceRpcOp(ApiAddNotifyKey_Request, ApiAddNotifyKey_Response),
    62: DceRpcOp(ApiReAddNotifyNode_Request, ApiReAddNotifyNode_Response),
    63: DceRpcOp(ApiReAddNotifyGroup_Request, ApiReAddNotifyGroup_Response),
    64: DceRpcOp(ApiReAddNotifyResource_Request, ApiReAddNotifyResource_Response),
    65: DceRpcOp(ApiGetNotify_Request, ApiGetNotify_Response),
    66: DceRpcOp(ApiOpenNode_Request, ApiOpenNode_Response),
    67: DceRpcOp(ApiCloseNode_Request, ApiCloseNode_Response),
    68: DceRpcOp(ApiGetNodeState_Request, ApiGetNodeState_Response),
    69: DceRpcOp(ApiPauseNode_Request, ApiPauseNode_Response),
    70: DceRpcOp(ApiResumeNode_Request, ApiResumeNode_Response),
    71: DceRpcOp(ApiEvictNode_Request, ApiEvictNode_Response),
    72: DceRpcOp(ApiNodeResourceControl_Request, ApiNodeResourceControl_Response),
    73: DceRpcOp(ApiResourceControl_Request, ApiResourceControl_Response),
    74: DceRpcOp(
        ApiNodeResourceTypeControl_Request, ApiNodeResourceTypeControl_Response
    ),
    75: DceRpcOp(ApiResourceTypeControl_Request, ApiResourceTypeControl_Response),
    76: DceRpcOp(ApiNodeGroupControl_Request, ApiNodeGroupControl_Response),
    77: DceRpcOp(ApiGroupControl_Request, ApiGroupControl_Response),
    78: DceRpcOp(ApiNodeNodeControl_Request, ApiNodeNodeControl_Response),
    79: DceRpcOp(ApiNodeControl_Request, ApiNodeControl_Response),
    # 80: Opnum80NotUsedOnWire,
    81: DceRpcOp(ApiOpenNetwork_Request, ApiOpenNetwork_Response),
    82: DceRpcOp(ApiCloseNetwork_Request, ApiCloseNetwork_Response),
    83: DceRpcOp(ApiGetNetworkState_Request, ApiGetNetworkState_Response),
    84: DceRpcOp(ApiSetNetworkName_Request, ApiSetNetworkName_Response),
    85: DceRpcOp(ApiCreateNetworkEnum_Request, ApiCreateNetworkEnum_Response),
    86: DceRpcOp(ApiGetNetworkId_Request, ApiGetNetworkId_Response),
    87: DceRpcOp(
        ApiSetNetworkPriorityOrder_Request, ApiSetNetworkPriorityOrder_Response
    ),
    88: DceRpcOp(ApiNodeNetworkControl_Request, ApiNodeNetworkControl_Response),
    89: DceRpcOp(ApiNetworkControl_Request, ApiNetworkControl_Response),
    90: DceRpcOp(ApiAddNotifyNetwork_Request, ApiAddNotifyNetwork_Response),
    91: DceRpcOp(ApiReAddNotifyNetwork_Request, ApiReAddNotifyNetwork_Response),
    92: DceRpcOp(ApiOpenNetInterface_Request, ApiOpenNetInterface_Response),
    93: DceRpcOp(ApiCloseNetInterface_Request, ApiCloseNetInterface_Response),
    94: DceRpcOp(ApiGetNetInterfaceState_Request, ApiGetNetInterfaceState_Response),
    95: DceRpcOp(ApiGetNetInterface_Request, ApiGetNetInterface_Response),
    96: DceRpcOp(ApiGetNetInterfaceId_Request, ApiGetNetInterfaceId_Response),
    97: DceRpcOp(
        ApiNodeNetInterfaceControl_Request, ApiNodeNetInterfaceControl_Response
    ),
    98: DceRpcOp(ApiNetInterfaceControl_Request, ApiNetInterfaceControl_Response),
    99: DceRpcOp(ApiAddNotifyNetInterface_Request, ApiAddNotifyNetInterface_Response),
    100: DceRpcOp(
        ApiReAddNotifyNetInterface_Request, ApiReAddNotifyNetInterface_Response
    ),
    101: DceRpcOp(ApiCreateNodeEnum_Request, ApiCreateNodeEnum_Response),
    102: DceRpcOp(ApiGetClusterVersion2_Request, ApiGetClusterVersion2_Response),
    103: DceRpcOp(ApiCreateResTypeEnum_Request, ApiCreateResTypeEnum_Response),
    104: DceRpcOp(ApiBackupClusterDatabase_Request, ApiBackupClusterDatabase_Response),
    105: DceRpcOp(ApiNodeClusterControl_Request, ApiNodeClusterControl_Response),
    106: DceRpcOp(ApiClusterControl_Request, ApiClusterControl_Response),
    107: DceRpcOp(ApiUnblockGetNotifyCall_Request, ApiUnblockGetNotifyCall_Response),
    108: DceRpcOp(
        ApiSetServiceAccountPassword_Request, ApiSetServiceAccountPassword_Response
    ),
}
register_dcerpc_interface(
    name="clusapi",
    uuid=uuid.UUID("b97db8b2-4c63-11cf-bff6-08002be23f2f"),
    version="2.0",
    opnums=CLUSAPI_OPNUMS,
)



### ms-raiw_winsif2.idl
"""
RPC definitions for the following interfaces:
- winsi2 (v1.0): 811109bf-a4e1-11d1-ab54-00a0c91e9b45
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRSignedLongField,
    register_dcerpc_interface,
)


class PWINSINTF_ADD_T(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRByteField("Type", 0),
        NDRIntField("Len", 0),
        NDRIntField("IPAdd", 0),
    ]


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class R_WinsTombstoneDbRecs_Request(NDRPacket):
    fields_desc = [
        NDRLongField("ServerHdl", 0),
        NDRPacketField("pWinsAdd", PWINSINTF_ADD_T(), PWINSINTF_ADD_T),
        NDRPacketField("MinVersNo", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("MaxVersNo", LARGE_INTEGER(), LARGE_INTEGER),
    ]


class R_WinsTombstoneDbRecs_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_WinsCheckAccess_Request(NDRPacket):
    fields_desc = [NDRLongField("ServerHdl", 0)]


class R_WinsCheckAccess_Response(NDRPacket):
    fields_desc = [NDRIntField("Access", 0), NDRIntField("status", 0)]


WINSI2_OPNUMS = {
    0: DceRpcOp(R_WinsTombstoneDbRecs_Request, R_WinsTombstoneDbRecs_Response),
    1: DceRpcOp(R_WinsCheckAccess_Request, R_WinsCheckAccess_Response),
}
register_dcerpc_interface(
    name="winsi2",
    uuid=uuid.UUID("811109bf-a4e1-11d1-ab54-00a0c91e9b45"),
    version="1.0",
    opnums=WINSI2_OPNUMS,
)



### ms-dltw.idl
"""
RPC definitions for the following interfaces:
- trkwks (v1.2): 300f3532-38cc-11d0-a3f0-0020af6b0add
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    register_dcerpc_interface,
)


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class CVolumeId(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRPacketField("_volume", GUID(), GUID)]


class CObjId(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRPacketField("_object", GUID(), GUID)]


class CDomainRelativeObjId(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("_volume", CVolumeId(), CVolumeId),
        NDRPacketField("_object", CObjId(), CObjId),
    ]


class CMachineId(NDRPacket):
    fields_desc = [StrFixedLenField("_szMachine", "", length=16)]


class LnkSearchMachine_Request(NDRPacket):
    fields_desc = [
        NDRIntField("Restrictions", 0),
        NDRPacketField("pdroidBirthLast", CDomainRelativeObjId(), CDomainRelativeObjId),
        NDRPacketField("pdroidLast", CDomainRelativeObjId(), CDomainRelativeObjId),
    ]


class LnkSearchMachine_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pdroidBirthNext", CDomainRelativeObjId(), CDomainRelativeObjId),
        NDRPacketField("pdroidNext", CDomainRelativeObjId(), CDomainRelativeObjId),
        NDRPacketField("pmcidNext", CMachineId(), CMachineId),
        NDRConfVarStrLenFieldUtf16("ptszPath", "", size_is=lambda pkt: 261),
        NDRIntField("status", 0),
    ]


TRKWKS_OPNUMS = {  # 0: Opnum0NotUsedOnWire,
    # 1: Opnum1NotUsedOnWire,
    # 2: Opnum2NotUsedOnWire,
    # 3: Opnum3NotUsedOnWire,
    # 4: Opnum4NotUsedOnWire,
    # 5: Opnum5NotUsedOnWire,
    # 6: Opnum6NotUsedOnWire,
    # 7: Opnum7NotUsedOnWire,
    # 8: Opnum8NotUsedOnWire,
    # 9: Opnum9NotUsedOnWire,
    # 10: Opnum10NotUsedOnWire,
    # 11: Opnum11NotUsedOnWire,
    12: DceRpcOp(LnkSearchMachine_Request, LnkSearchMachine_Response)
}
register_dcerpc_interface(
    name="trkwks",
    uuid=uuid.UUID("300f3532-38cc-11d0-a3f0-0020af6b0add"),
    version="1.2",
    opnums=TRKWKS_OPNUMS,
)



### ms-rsp_initshutdown.idl
"""
RPC definitions for the following interfaces:
- InitShutdown (v1.0): 894de0c0-0d55-11d3-a322-00c04fa321a1
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    register_dcerpc_interface,
)


class PREG_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class BaseInitiateShutdown_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRShortField("ServerName", 0)),
        NDRFullPointerField(
            NDRPacketField("lpMessage", PREG_UNICODE_STRING(), PREG_UNICODE_STRING)
        ),
        NDRIntField("dwTimeout", 0),
        NDRByteField("bForceAppsClosed", 0),
        NDRByteField("bRebootAfterShutdown", 0),
    ]


class BaseInitiateShutdown_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class BaseAbortShutdown_Request(NDRPacket):
    fields_desc = [NDRFullPointerField(NDRShortField("ServerName", 0))]


class BaseAbortShutdown_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class BaseInitiateShutdownEx_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRShortField("ServerName", 0)),
        NDRFullPointerField(
            NDRPacketField("lpMessage", PREG_UNICODE_STRING(), PREG_UNICODE_STRING)
        ),
        NDRIntField("dwTimeout", 0),
        NDRByteField("bForceAppsClosed", 0),
        NDRByteField("bRebootAfterShutdown", 0),
        NDRIntField("dwReason", 0),
    ]


class BaseInitiateShutdownEx_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


INITSHUTDOWN_OPNUMS = {
    0: DceRpcOp(BaseInitiateShutdown_Request, BaseInitiateShutdown_Response),
    1: DceRpcOp(BaseAbortShutdown_Request, BaseAbortShutdown_Response),
    2: DceRpcOp(BaseInitiateShutdownEx_Request, BaseInitiateShutdownEx_Response),
}
register_dcerpc_interface(
    name="InitShutdown",
    uuid=uuid.UUID("894de0c0-0d55-11d3-a322-00c04fa321a1"),
    version="1.0",
    opnums=INITSHUTDOWN_OPNUMS,
)



### ms-msrp_msgsvc.idl
"""
RPC definitions for the following interfaces:
- msgsvc (v1.0): 17FDD703-1827-4E34-79D4-24A55C53BB37
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRUnionField,
    register_dcerpc_interface,
)


class NetrMessageNameAdd_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("MsgName", ""),
    ]


class NetrMessageNameAdd_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class LPMSG_INFO_0(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("msgi0_name", ""), deferred=True
        )
    ]


class LPMSG_INFO_0_CONTAINER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntriesRead", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Buffer",
                [LPMSG_INFO_0()],
                LPMSG_INFO_0,
                size_is=lambda pkt: pkt.EntriesRead,
            ),
            deferred=True,
        ),
    ]


class LPMSG_INFO_1(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("msgi1_name", ""), deferred=True
        ),
        NDRIntField("msgi1_forward_flag", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("msgi1_forward", ""), deferred=True
        ),
    ]


class LPMSG_INFO_1_CONTAINER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntriesRead", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Buffer",
                [LPMSG_INFO_1()],
                LPMSG_INFO_1,
                size_is=lambda pkt: pkt.EntriesRead,
            ),
            deferred=True,
        ),
    ]


class LPMSG_ENUM_STRUCT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "MsgInfo", LPMSG_INFO_0_CONTAINER(), LPMSG_INFO_0_CONTAINER
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 0),
                        (lambda _, val: val.tag == 0),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "MsgInfo", LPMSG_INFO_1_CONTAINER(), LPMSG_INFO_1_CONTAINER
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
            ],
            StrFixedLenField("MsgInfo", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class NetrMessageNameEnum_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRPacketField("InfoStruct", LPMSG_ENUM_STRUCT(), LPMSG_ENUM_STRUCT),
        NDRIntField("PrefMaxLen", 0),
        NDRFullPointerField(NDRIntField("ResumeHandle", 0)),
    ]


class NetrMessageNameEnum_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("InfoStruct", LPMSG_ENUM_STRUCT(), LPMSG_ENUM_STRUCT),
        NDRIntField("TotalEntries", 0),
        NDRFullPointerField(NDRIntField("ResumeHandle", 0)),
        NDRIntField("status", 0),
    ]


class NetrMessageNameGetInfo_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("MsgName", ""),
        NDRIntField("Level", 0),
    ]


class NetrMessageNameGetInfo_Response(NDRPacket):
    fields_desc = [
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPMSG_INFO_0(), LPMSG_INFO_0)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 0),
                        (lambda _, val: val.tag == 0),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPMSG_INFO_1(), LPMSG_INFO_1)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
            ],
            StrFixedLenField("InfoStruct", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
        NDRIntField("status", 0),
    ]


class NetrMessageNameDel_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("MsgName", ""),
    ]


class NetrMessageNameDel_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


MSGSVC_OPNUMS = {
    0: DceRpcOp(NetrMessageNameAdd_Request, NetrMessageNameAdd_Response),
    1: DceRpcOp(NetrMessageNameEnum_Request, NetrMessageNameEnum_Response),
    2: DceRpcOp(NetrMessageNameGetInfo_Request, NetrMessageNameGetInfo_Response),
    3: DceRpcOp(NetrMessageNameDel_Request, NetrMessageNameDel_Response),
}
register_dcerpc_interface(
    name="msgsvc",
    uuid=uuid.UUID("17FDD703-1827-4E34-79D4-24A55C53BB37"),
    version="1.0",
    opnums=MSGSVC_OPNUMS,
)
